package com.csversion2;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import javax.microedition.khronos.opengles.GL10;

import com.csversion2.R;

import android.content.Context;
import android.content.res.Resources;
  
public class Arrow {
//	private static final String TAG = "FLECHA";
	private FloatBuffer vertexBuffer;  // Buffer for vertex-array
	private FloatBuffer colorBuffer;   // Buffer for color-array
	private ByteBuffer indexBuffer;    // Buffer for index-array
	private FloatBuffer normalsBuffer;
	private float[] vertices;
	private float[] normales;
	private byte[] indices;
	private float[] colors;
  
   // Constructor - Set up the buffers
	public Arrow(Context context) {
		
		int contVertices = 0;
		int contNormales = 0;
		int contFaces = 0;
		
		//tengo que abrir el fichero obj y crear mis variables de vertices y caras
		// parser del obj para crear la flecha 3D
		try {
			   Resources res = context.getResources();
			   InputStream fraw = res.openRawResource(R.raw.flecha_obj);
			   BufferedReader brin = new BufferedReader(new InputStreamReader(fraw));
			   String linea = null;
			   while((linea = brin.readLine()) != null){
				   if(linea.startsWith("v ")){
					   contVertices += 1;
				   }
				   else if(linea.startsWith("vn ")){
					   contNormales += 1;
				   }
				   else if(linea.startsWith("f ")){
					   contFaces += 1;
				   }
			   }
			   fraw.close();
			   
			   vertices = new float[contVertices * 3];
			   normales = new float[contNormales * 3];
			   colors = new float[contVertices * 4];
			   indices = new byte[contFaces * 3];
			   String[] s = new String[4];
			   String[] aux = new String[2];
			   int i = 0;
			   int j = 0;
			   int h = 0;
			   
			   for(int k = 0; k < contVertices*4; k=k+4){
				   colors[k] = 0.0f;
				   colors[k+1] = 0.0f;
				   colors[k+2] = 1.0f;
				   colors[k+3] = 1.0f;
			   }
			   
			   fraw = res.openRawResource(R.raw.flecha_obj);
			   brin = new BufferedReader(new InputStreamReader(fraw));
			   while((linea = brin.readLine()) != null){
				   if(linea.startsWith("v ")){
					   s = linea.split(" ");
					   vertices[i] = Float.parseFloat(s[1]);
					   vertices[i+1] = Float.parseFloat(s[2]);
					   vertices[i+2] = Float.parseFloat(s[3]);
					   i += 3;
				   }
				   else if(linea.startsWith("vn ")){
					   s = linea.split(" ");
					   normales[h] = Float.parseFloat(s[1]);
					   normales[h+1] = Float.parseFloat(s[2]);
					   normales[h+2] = Float.parseFloat(s[3]);
					   h += 3;
				   }
				   else if(linea.startsWith("f ")){
					   s = linea.split(" ");
					   aux = s[1].split("/");
					   indices[j] = (byte) (Integer.parseInt(aux[0]) - 1);
					   aux = s[2].split("/");
					   indices[j+1] = (byte) (Integer.parseInt(aux[0]) - 1);
					   aux = s[3].split("/");
					   indices[j+2] = (byte) (Integer.parseInt(aux[0]) - 1);
					   j += 3;
				   }
			   }
			   fraw.close();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	   
      // Setup vertex-array buffer. Vertices in float. An float has 4 bytes
      ByteBuffer vbb = ByteBuffer.allocateDirect(vertices.length * 4);
      vbb.order(ByteOrder.nativeOrder()); // Use native byte order
      vertexBuffer = vbb.asFloatBuffer(); // Convert from byte to float
      vertexBuffer.put(vertices);         // Copy data into buffer
      vertexBuffer.position(0);           // Rewind
      
      ByteBuffer nbb = ByteBuffer.allocateDirect(normales.length * 4);
      nbb.order(ByteOrder.nativeOrder()); // Use native byte order
      normalsBuffer = nbb.asFloatBuffer(); // Convert from byte to float
      normalsBuffer.put(normales);         // Copy data into buffer
      normalsBuffer.position(0);           // Rewind
  
      // Setup color-array buffer. Colors in float. An float has 4 bytes
      ByteBuffer cbb = ByteBuffer.allocateDirect(colors.length * 4);
      cbb.order(ByteOrder.nativeOrder());
      colorBuffer = cbb.asFloatBuffer();
      colorBuffer.put(colors);
      colorBuffer.position(0);
  
      // Setup index-array buffer. Indices in byte.
      indexBuffer = ByteBuffer.allocateDirect(indices.length);
      indexBuffer.put(indices);
      indexBuffer.position(0);
	}
  
   // Draw the shape
	public void draw(GL10 gl) {
		
		gl.glFrontFace(GL10.GL_CCW);    // Front face in counter-clockwise orientation
		gl.glEnable(GL10.GL_CULL_FACE); // Enable cull face
		gl.glEnable(GL10.GL_DEPTH_TEST);
		
//		gl.glEnable(GL10.GL_LIGHTING);
  
		// Enable arrays and define their buffers
		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		gl.glVertexPointer(3, GL10.GL_FLOAT, 0, vertexBuffer);
		
//		gl.glEnableClientState(GL10.GL_NORMAL_ARRAY);
//		gl.glNormalPointer(3, GL10.GL_FLOAT, normalsBuffer);
		
		//gl.glDisableClientState(GL10.GL_COLOR_ARRAY);
		gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
		gl.glColorPointer(4, GL10.GL_FLOAT, 0, colorBuffer);
		//gl.glColor4f(0.0f, 0.0f, 1.0f, 1.0f);

		gl.glScalef(0.12f, 0.12f, 0.12f);
		//gl.glRotatef(-90.0f, 0.0f, 0.0f, 1.0f);
		gl.glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
		//gl.glTranslatef(0.0f, -5.0f, -9.0f);
      
		gl.glDrawElements(GL10.GL_TRIANGLES, indices.length, GL10.GL_UNSIGNED_BYTE, indexBuffer);
      
		gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);
		gl.glDisableClientState(GL10.GL_COLOR_ARRAY);
//		gl.glDisableClientState(GL10.GL_NORMAL_ARRAY);
	}
}