package com.csversion2;

import java.util.List;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import com.csversion2.data.ARData;
import com.csversion2.ui.Marker;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.hardware.GeomagneticField;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.location.Location;
import android.opengl.GLSurfaceView;
import android.os.IBinder;

public class CompassRenderer extends Service implements GLSurfaceView.Renderer, SensorEventListener{

//	private static final String TAG = "COMPASS";
	private Arrow arrow;
    private float mAccelerometerValues[] = new float[3];
	private float mMagneticValues[] = new float[3];
    private float rotationMatrix[] = new float[16];
    private float remappedRotationMatrix[] = new float[16];
    private float orientacion[] = new float[3];
    private Location loc, target;

	public CompassRenderer(Context context) {
	    this.arrow = new Arrow(context);
		this.loc = ARData.getCurrentLocation();
	}
	
	public void onDrawFrame(GL10 gl) {
	    // Get rotation matrix from the sensor
	    SensorManager.getRotationMatrix(rotationMatrix, null, mAccelerometerValues, mMagneticValues);
	    // As the documentation says, we are using the device as a compass in landscape mode
	    SensorManager.remapCoordinateSystem(rotationMatrix, SensorManager.AXIS_Y, SensorManager.AXIS_MINUS_X, remappedRotationMatrix);
		SensorManager.getOrientation(remappedRotationMatrix, orientacion);
		
		this.loc = ARData.getCurrentLocation();
		
		List<Marker> markers = ARData.getRouteMarkers();
		Marker m = markers.get(0);
		double[] latlong = m.getLatLong();
		
		target = new Location(this.loc);
		target.setLatitude(latlong[0]);
		target.setLongitude(latlong[1]);
		
		// AZIMUTH -> EJE Z -> EJE Y
		float azimuth = orientacion[0];
		//System.out.println(azimuth);
		azimuth = azimuth * 360 / (2 * (float) Math.PI);
		GeomagneticField geoField = new GeomagneticField(
		             Double.valueOf(this.loc.getLatitude()).floatValue(),
		             Double.valueOf(this.loc.getLongitude()).floatValue(),
		             Double.valueOf(this.loc.getAltitude()).floatValue(),
		             System.currentTimeMillis());
		azimuth -= geoField.getDeclination(); // converts magnetic north into true north
		
		// si la distancia entre yo y mi siguiente punto de destino es menor a X metros, considero que he llegado
//		float distancia = loc.distanceTo(target);
		
		float bearing = this.loc.bearingTo(target); // (it's already in degrees)
		if (bearing < 0) {
			bearing = bearing + 360;
	    }
		
		float grados = azimuth - bearing;
		if (grados < 0) {
			grados = grados + 360;
	    }
		
	    // Clear color buffer
	    gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
	    gl.glMatrixMode(GL10.GL_MODELVIEW);
	    gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
	    gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
		// ----- Render the Arrow -----
		gl.glLoadIdentity();                 // Reset the model-view matrix
		gl.glTranslatef(0.0f, -5.0f, -9.0f);		
//		gl.glRotatef(azimuth + bearing, 0.0f, 1.0f, 0.0f);
		gl.glRotatef(grados, 0.0f, 1.0f, 0.0f);
		arrow.draw(gl);
	}
	
	public void onSurfaceChanged(GL10 gl, int width, int height) {
		if (height == 0){
			height = 1;   // To prevent divide by zero
		}
	      
		gl.glViewport(0, 0, width, height);
		/*
	     * Set our projection matrix. This doesn't have to be done
	    * each time we draw, but usually a new projection needs to
	    * be set when the viewport is resized.
	    */
		float ratio = (float) width / height;
		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();		// Reset
		gl.glFrustumf(-ratio, ratio, -1, 1, 1, 100);
	}
	
	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		
		//las de la flecha
		gl.glClearDepthf(1.0f);            // Set depth's clear-value to farthest
		gl.glEnable(GL10.GL_DEPTH_TEST);   // Enables depth-buffer for hidden surface removal
		gl.glDepthFunc(GL10.GL_LEQUAL);    // The type of depth testing to do
		gl.glShadeModel(GL10.GL_SMOOTH);   // Enable smooth shading of color
		
	    /*
	     * By default, OpenGL enables features that improve quality
	     * but reduce performance. One might want to tweak that
	     * especially on software renderer.
	     */
		gl.glDisable(GL10.GL_DITHER);
	
	    /*
	     * Some one-time OpenGL initialization can be made here
	     * probably based on features of this particular context
	     */
	    gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT, GL10.GL_FASTEST);
	    gl.glClearColor(0,0,0,0);
	}

    @Override
    public void onAccuracyChanged(Sensor arg0, int arg1) {
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
           synchronized (this) {
                    switch(event.sensor.getType()) {
                    case Sensor.TYPE_ACCELEROMETER:
                            //mAccelerometerValues = event.values.clone();
                            mAccelerometerValues[0]=(mAccelerometerValues[0]*8+event.values[0])*0.11112f;
                            mAccelerometerValues[1]=(mAccelerometerValues[1]*8+event.values[1])*0.11112f;
                            mAccelerometerValues[2]=(mAccelerometerValues[2]*8+event.values[2])*0.11112f;
                        break;
                    case Sensor.TYPE_MAGNETIC_FIELD:
                            //mMagneticValues = event.values.clone();
                    	mMagneticValues[0]=(mMagneticValues[0]*8+event.values[0])*0.11112f;
                    	mMagneticValues[1]=(mMagneticValues[1]*8+event.values[1])*0.11112f;
                    	mMagneticValues[2]=(mMagneticValues[2]*8+event.values[2])*0.11112f;
                        break;
                    default:
                            break;
                    }
           }
    }

	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}
}