package com.csversion2.data;

import java.util.ArrayList;
import java.util.List;

import android.content.res.Resources;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;

import com.csversion2.R;
import com.csversion2.ui.IconMarker;
import com.csversion2.ui.Marker;


/**
 * This class should be used as a example local data source. It is an example
 * of how to add data programatically. You can add data either programatically,
 * SQLite or through any other source.
 *
 * @author Justin Wetherell <phishman3579@gmail.com>
 */
public class LocalDataSource extends DataSource{
	
	private String DATABASE_NAME_POIS = "pois.db";
	private String DATABASE_NAME_ROUTE = "RutaPamplona.db";
	private static final String DATABASE_PATH = "/data/data/com.csversion2/databases/";
	
    private List<Marker> cachedMarkers = new ArrayList<Marker>();
    private List<Marker> cachedMarkersRoute = new ArrayList<Marker>();
    private static Bitmap icon = null;
    private Resources res = null;
   
    public LocalDataSource(Resources res, String ruta, String idioma) {
        if (res==null) throw new NullPointerException();
       this.res = res;
       this.DATABASE_NAME_ROUTE = ruta;
       this.DATABASE_NAME_POIS = idioma;
    }
   
    public List<Marker> getMarkers() {

		String myPath = DATABASE_PATH + DATABASE_NAME_POIS;
		SQLiteDatabase database = SQLiteDatabase.openDatabase(myPath, null, SQLiteDatabase.OPEN_READONLY);

		String sql = "SELECT * FROM Pois";
		Cursor c = database.rawQuery(sql, null);

		int tipo = 1;
		double latitud = 0, longitud = 0;
		String nombre = null, desc = null;

		c.moveToFirst();
		while (!c.isAfterLast()) {
			// 0 tipo 1 id 2 latitud 3 longitud 4 nombre 5 descripcion
			tipo = c.getInt(0);
			latitud = Double.parseDouble(c.getString(2));
			longitud = Double.parseDouble(c.getString(3));
			nombre = c.getString(4);
			desc = c.getString(5);
			
			if(tipo == 1){
				// albergue
				icon=BitmapFactory.decodeResource(this.res, R.drawable.inn);
			}
			else if (tipo == 2){
				// hotel
				icon=BitmapFactory.decodeResource(this.res, R.drawable.hotel);
			}
			else if (tipo == 3){
				// restaurante
				icon=BitmapFactory.decodeResource(this.res, R.drawable.restaurant);
			}
			else if (tipo == 4){
				// monumento...?
				icon=BitmapFactory.decodeResource(this.res, R.drawable.monument);
			}
			else if (tipo == 5){
				// informacin
				icon=BitmapFactory.decodeResource(this.res, R.drawable.info);
			}
			
			Marker m = new IconMarker(nombre, latitud, longitud, 0, Color.YELLOW, icon, desc);
	        cachedMarkers.add(m);
			
			c.moveToNext();
		}

		c.close();
		database.close();

        return cachedMarkers;
    }
    
    public List<Marker> getRouteMarkers() {

		String myPath = DATABASE_PATH + this.DATABASE_NAME_ROUTE;
		SQLiteDatabase database = SQLiteDatabase.openDatabase(myPath, null, SQLiteDatabase.OPEN_READONLY);
		
		int tam = this.DATABASE_NAME_ROUTE.length();
		String ruta = this.DATABASE_NAME_ROUTE.substring(0, tam-3);

		String sql = "SELECT * FROM " + ruta;
		Cursor c = database.rawQuery(sql, null);

		int id = 1;
		double latitud = 0, longitud = 0;

		c.moveToFirst();
		while (!c.isAfterLast()) {
			// 0 id 1 latitud 2 longitud
			id = c.getInt(0);
			latitud = Double.parseDouble(c.getString(1));
			longitud = Double.parseDouble(c.getString(2));
			
			// buscar icono de la ruta (tiene que ser pequeo)
			icon=BitmapFactory.decodeResource(this.res, R.drawable.inn);
			
			Marker m = new IconMarker("Punto " + id, latitud, longitud, 0, Color.BLUE, icon, "Punto " + id + " de la ruta");
			cachedMarkersRoute.add(m);
			
			c.moveToNext();
		}

		c.close();
		database.close();

        return cachedMarkersRoute;
    }
}